{% extends "base.html" %}

{% block title %}
	&raquo; Connection Agreements
{% endblock %}

{% block nav %}
	Connection Agreements
{% endblock %}

{% block content %}

	<table>
		<caption>Connection Agreements</caption>	
		<thead>
			<tr>
				<th>View</th>
				<th>Start Date</th>
				<th>Finish Date</th>
				<th>Document</th>
			</tr>
		</thead>
		<tbody>
			{% for ca in cas %}
				{% set props = ca.properties %}
				<tr>
					<td><a href="/e/cas/{{ca.id}}">View</a></td>
					<td>{{ca.start_date|hh_format}}</td>
					<td>{{ca.finish_date|hh_format}}</td>
					<td>
						{% if 'url' in props %}
							<a href="{{props['url']}}" target="_blank" rel="noopener noreferrer">Document</a>
						{% endif %}
					</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% endblock %}
