{% extends "base.html" %}

{% block title %}
	&raquo; Dashboard
{% endblock %}

{% block nav %}
	Dashboard
{% endblock %}

{% block content %}

	<ul>
		<li><a href="/e/dashboard/ecoes">ECOES Comparison</a></a>
		<li><a href="/e/dashboard/rolling_year">Rolling Year £ and kWh</a></a>
	</ul>

{% endblock %}
