{% extends "base.html" %}

{% block title %}
	&raquo; CSV Supplies HH Duration
{% endblock %}

{% block nav %}
	CSV Supplies HH Duration
{% endblock %}

{% block content %}
	<form action="/reports/169" method='post'>
		<fieldset>
			<legend>48 Column Format</legend>	
			<label>Import / Export</label>
			<select name="imp_related">
				{{input_option('imp_related', 'true', 'Import')}}
				{{input_option('imp_related', 'false', 'Export')}}
			</select>
			<label>Channel Type</label>
			<select name="channel_type">
				{{input_option('channel_type', 'ACTIVE', 'ACTIVE')}}
				{{input_option('channel_type', 'REACTIVE_IMP', 'REACTIVE_IMP')}}
				{{input_option('channel_type', 'REACTIVE_EXP', 'REACTIVE_EXP')}}
			</select>
			<label>From</label> {{input_date('start', start_date, resolution='day')}}
			<label>To</label> {{input_date('finish', finish_date, resolution='day')}}
			<label>Zipped?</label> {{input_checkbox('is_zipped', False)}}
			<label>MPAN Cores</label>
			{{input_textarea(
				'mpan_cores', '', 5, 40,
				placeholder='One on each line, includes all supplies if left blank'
				)}}    						
			<input type="submit" value="Download">
		</fieldset>
	</form>

	<form action="/reports/187" method='post'>
		<fieldset>
			<legend>HH Per Row Format</legend>	
			<label>From</label> {{input_date('start', start_date)}}
			<label>To</label> {{input_date('finish', finish_date)}}
			<label>Zipped</label> {{input_checkbox('is_zipped', False)}}
			<label>MPAN Cores</label>
			{{input_textarea(
				'mpan_cores', '', 5, 40,
				placeholder='One on each line, includes all supplies if left blank'
				)}}
			<input type="submit" value="Download">
		</fieldset>
	</form>
			
{% endblock %}
