{% extends "base.html" %}

{% block title %}
	&raquo; CSV Register Reads
{% endblock %}

{% block nav %}
	CSV Register Reads
{% endblock %}

{% block content %}
	<form action="/reports/219">
		<fieldset>
			<legend>Download</legend>	
			<label>Months</label> {{input_text('months', 1, 2, 2)}}
			<label>Finish Month</label> {{input_date('end', init, resolution='month')}}
			<input type="submit" value="Download">
		</fieldset>
	</form>

	<p>
		A register read has both a previous and present reading. A read will appear in the
	 	report if either the previous or present reading date falls within the reporting
	 	period. The reason is that it may be that a reading only appears as a previous
	 	reading (if the supply is new for example).
	</p>

{% endblock %}
