{% extends "base.html" %}

{% block title %}
	&raquo; CSV Bills
{% endblock %}

{% block nav %}
	CSV Bills
{% endblock %}

{% block content %}

	<form action="/reports/27">
		<fieldset>
			<legend>Download</legend>	
			<label>Months</label> {{input_text('months', 1, 2, 2)}}
			<label>Final Month</label>
			{{input_date('end', init, resolution='month')}}
			<input type="submit" value="Download">
		</fieldset>
	</form>

{% endblock %}
